package obj

import
  "reflect"

type
  Equaler interface { // Mit x ist immer das aufrufende Objekt gemeint.

// Liefert genau dann true,
// wenn x vom gleichen Typ wie y ist und mit y übereinstimmt.
  Eq (y Any) bool

// Liefert eine Kopie von x.
// (Konsequenz: x.Eq (x.Clone()) == true.)
  Clone() Any

// Wenn x vom gleichen Typ wie x ist, stimmt x mit y überein. (Konsequenz: x.Eq(y).)
// y ist unverändert.
  Copy (y Any)
}

// Liefert genau dann true, wenn a und b vom gleichen Typ sind.
func TypeEq (a, b Any) bool {
  return reflect.TypeOf(a) == reflect.TypeOf(b)
}

func Eq (a, b Any) bool {
  if a == nil { return b == nil }
  if ! TypeEq (a, b) {
    return false
  }
  if x, ok:= a.(Object); ok {
    return x.Eq (b.(Object))
  }
  if x, ok:= a.([]byte); ok {
    for i:= 0; i < len (x); i++ {
      if a.([]byte)[i] != b.([]byte)[i] {
        return false
      }
      return true
    }
  }
  if Atomic (a) {
    return a == b
  }
  return reflect.DeepEqual (a, b)
}

func Clone (a Any) Any {
  if a == nil {
    return nil
  }
  if Atomic (a) {
    return a
  }
  switch a.(type) { case Object:
    return a.(Object).Clone()
  case []byte:
    b:= make ([]byte, len (a.([]byte)))
    copy (b, a.([]byte))
    return b
  default:
    // TODO
  }
  return nil
}
