package r3num

import (
  "math"
  "strconv"
  . "gokurs/obj"
  . "gokurs/num"
)
type
  root3number struct {
                a, b int
                     }

func New() *root3number {
  return new(root3number)
}

func (x *root3number) imp (Y Any) *root3number {
  y, ok:= Y.(*root3number)
  if ! ok { panic("Typfehler") }
  return y
}

func (x *root3number) Empty() bool {
  return x.a == 0 && x.b == 0
}

func (x *root3number) Clr() {
  x.a, x.b = 0, 0
}

func (x *root3number) Copy (Y Any) {
  y:= x.imp(Y)
  x.a, x.b = y.a, y.b
}

func (x *root3number) Clone() Any {
  y:= New()
  y.Copy (x)
  return y
}

func (x *root3number) Get() (int, int) {
  return x.a, x.b
}

func (x *root3number) Add (Y Adder) {
  y:= x.imp(Y)
  x.a += y.a
  x.b += y.b
}

func (x *root3number) Sub (Y Adder) {
  y:= x.imp(Y)
  x.a -= y.a
  x.b -= y.b
}

func (x *root3number) Sum (Y Adder) Adder {
  a, b:= New(), x.imp(Y)
  a.a, a.b = x.a + b.a, x.b + b.b
  return a
}

func (x *root3number) Mul (Y Multiplier) {
  y:= x.imp(Y)
  x.a, x.b = x.a * y.a + x.b * y.b * 3, x.a * y.b + x.b * y.a
}

func (x *root3number) Prod (Y, Z Multiplier) {
  y, z:= x.imp(Y), x.imp(Z)
  x.a, x.b = y.a * z.a + y.b * z.b * 3, y.a * z.b + y.b * z.a
}

func (x *root3number) Val() float64 {
  return float64(x.a) + float64(x.b) * math.Sqrt(3)
}

func (x *root3number) Less (Y Any) bool {
  return x.Val() < x.imp(Y).Val()
}

func (x *root3number) Eq (Y Any) bool {
  y:= x.imp(Y)
  return x.a == y.a && x.b == y.b
}

func (x *root3number) Set (a, b int) {
  x.a, x.b = a, b
}

func (x *root3number) String() string {
  s:= ""; if x.b >= 0 { s = "+" }
  return strconv.Itoa (x.a) + s + strconv.Itoa (x.b) + "*sqrt(3)"
}
